=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2011-07-12 23:34:13 +0000
+++ drizzled/drizzled.cc	2011-07-18 02:56:15 +0000
@@ -455,20 +455,26 @@
 }
 
 
-void unireg_abort(int exit_code)
+void unireg_exit()
 {
-  if (exit_code)
-  {
-    errmsg_printf(error::ERROR, _("Aborting"));
-  }
-  else if (opt_help)
+  if (opt_help)
   {
     usage();
   }
 
-  clean_up(!opt_help && exit_code);
-  internal::my_end();
-  exit(exit_code);
+  internal::my_end();
+  exit(EXIT_SUCCESS);
+}
+
+void unireg_actual_abort(const char *file, int line, const char *func, const std::string& message)
+{
+  std::stringstream temp;
+  temp << _("Aborting:") << "\"" << message << "\"" << " at " << file << ":" << line << " in " << func << "()";
+  errmsg_printf(error::ERROR, temp.str().c_str());
+
+  clean_up(opt_help == false);
+  internal::my_end();
+  exit(EXIT_FAILURE);
 }
 
 
@@ -525,30 +531,29 @@
   }
   if (not user)
   {
-      errmsg_printf(error::ERROR, _("Fatal error: Please read \"Security\" section of "
-                                    "the manual to find out how to run drizzled as root"));
-    unireg_abort(1);
+    unireg_abort << _("Fatal error: Please read \"Security\" section of the manual to find out how to run drizzled as root");
   }
 
   if (not strcmp(user, "root"))
+  {
     return NULL;                        // Avoid problem with dynamic libraries
+  }
 
-  if (!(tmp_user_info= getpwnam(user)))
+  if ((tmp_user_info= getpwnam(user)) == NULL)
   {
     // Allow a numeric uid to be used
     const char *pos;
     for (pos= user; my_isdigit(&my_charset_utf8_general_ci,*pos); pos++) ;
     if (*pos)                                   // Not numeric id
       goto err;
-    if (!(tmp_user_info= getpwuid(atoi(user))))
+
+    if ((tmp_user_info= getpwuid(atoi(user))) == NULL)
       goto err;
   }
   return tmp_user_info;
 
 err:
-  errmsg_printf(error::ERROR, _("Fatal error: Can't change to run as user '%s' ;  "
-                    "Please check that the user exists!\n"),user);
-  unireg_abort(1);
+  unireg_abort << "Fatal error: Can't change to run as user '" << user << "' ;  Please check that the user exists!";
 
 #ifdef PR_SET_DUMPABLE
   if (getDebug().test(debug::CORE_ON_SIGNAL))
@@ -558,11 +563,7 @@
   }
 #endif
 
-/* Sun Studio 5.10 doesn't like this line.  5.9 requires it */
-#if defined(__SUNPRO_CC) && (__SUNPRO_CC <= 0x590)
   return NULL;
-#endif
-
 }
 
 void set_user(const char *user, passwd *user_info_arg)
@@ -571,13 +572,11 @@
   initgroups(user, user_info_arg->pw_gid);
   if (setgid(user_info_arg->pw_gid) == -1)
   {
-    sql_perror(_("Set process group ID failed"));
-    unireg_abort(1);
+    unireg_abort << _("Set process group ID failed ") << strerror(errno); 
   }
   if (setuid(user_info_arg->pw_uid) == -1)
   {
-    sql_perror(_("Set process user ID failed"));
-    unireg_abort(1);
+    unireg_abort << _("Set process user ID failed") << strerror(errno);
   }
 }
 
@@ -586,10 +585,9 @@
 /** Change root user if started with @c --chroot . */
 static void set_root(const char *path)
 {
-  if ((chroot(path) == -1) || !chdir("/"))
+  if ((chroot(path) == -1) or chdir("/") == 0)
   {
-    sql_perror(_("Process chroot failed"));
-    unireg_abort(1);
+    unireg_abort << _("Process chroot failed");
   }
 }
 
@@ -688,8 +686,7 @@
   global_system_variables.auto_increment_increment= 1;
   if (in_auto_increment_increment < 1 || in_auto_increment_increment > UINT64_MAX)
   {
-    cout << _("Error: Invalid Value for auto_increment_increment");
-    exit(-1);
+    unireg_abort << _("Invalid Value for auto_increment_increment");
   }
   global_system_variables.auto_increment_increment= in_auto_increment_increment;
 }
@@ -699,8 +696,7 @@
   global_system_variables.auto_increment_offset= 1;
   if (in_auto_increment_offset < 1 || in_auto_increment_offset > UINT64_MAX)
   {
-    cout << _("Error: Invalid Value for auto_increment_offset");
-    exit(-1);
+    unireg_abort << _("Invalid Value for auto_increment_offset");
   }
   global_system_variables.auto_increment_offset= in_auto_increment_offset;
 }
@@ -710,8 +706,7 @@
   global_system_variables.completion_type= 0;
   if (in_completion_type > 2)
   {
-    cout << _("Error: Invalid Value for completion_type");
-    exit(-1);
+    unireg_abort << _("Invalid Value for completion_type");
   }
   global_system_variables.completion_type= in_completion_type;
 }
@@ -722,8 +717,7 @@
   global_system_variables.div_precincrement= 4;
   if (in_div_precincrement > DECIMAL_MAX_SCALE)
   {
-    cout << _("Error: Invalid Value for div-precision-increment");
-    exit(-1);
+    unireg_abort << _("Invalid Value for div-precision-increment");
   }
   global_system_variables.div_precincrement= in_div_precincrement;
 }
@@ -733,8 +727,7 @@
   global_system_variables.group_concat_max_len= 1024;
   if (in_group_concat_max_len > ULONG_MAX || in_group_concat_max_len < 4)
   {
-    cout << _("Error: Invalid Value for group_concat_max_len");
-    exit(-1);
+    unireg_abort << _("Invalid Value for group_concat_max_len");
   }
   global_system_variables.group_concat_max_len= in_group_concat_max_len;
 }
@@ -744,8 +737,7 @@
   global_system_variables.join_buff_size= (128*1024L);
   if (in_join_buffer_size < IO_SIZE*2 || in_join_buffer_size > ULONG_MAX)
   {
-    cout << _("Error: Invalid Value for join_buffer_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for join_buffer_size");
   }
   in_join_buffer_size-= in_join_buffer_size % IO_SIZE;
   global_system_variables.join_buff_size= in_join_buffer_size;
@@ -756,8 +748,7 @@
   global_system_variables.max_allowed_packet= (64*1024*1024L);
   if (in_max_allowed_packet < 1024 || in_max_allowed_packet > 1024*1024L*1024L)
   {
-    cout << _("Error: Invalid Value for max_allowed_packet");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_allowed_packet");
   }
   in_max_allowed_packet-= in_max_allowed_packet % 1024;
   global_system_variables.max_allowed_packet= in_max_allowed_packet;
@@ -768,8 +759,7 @@
   global_system_variables.max_error_count= DEFAULT_ERROR_COUNT;
   if (in_max_error_count > 65535)
   {
-    cout << _("Error: Invalid Value for max_error_count");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_error_count");
   }
   global_system_variables.max_error_count= in_max_error_count;
 }
@@ -779,8 +769,7 @@
   global_system_variables.max_heap_table_size= (16*1024*1024L);
   if (in_max_heap_table_size < 16384 || in_max_heap_table_size > MAX_MEM_TABLE_SIZE)
   {
-    cout << _("Error: Invalid Value for max_heap_table_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_heap_table_size");
   }
   in_max_heap_table_size-= in_max_heap_table_size % 1024;
   global_system_variables.max_heap_table_size= in_max_heap_table_size;
@@ -791,8 +780,7 @@
   global_system_variables.min_examined_row_limit= 0;
   if (in_min_examined_row_limit > ULONG_MAX)
   {
-    cout << _("Error: Invalid Value for min_examined_row_limit");
-    exit(-1);
+    unireg_abort << _("Invalid Value for min_examined_row_limit");
   }
   global_system_variables.min_examined_row_limit= in_min_examined_row_limit;
 }
@@ -802,8 +790,7 @@
   global_system_variables.max_join_size= INT32_MAX;
   if ((uint64_t)in_max_join_size < 1 || (uint64_t)in_max_join_size > INT32_MAX)
   {
-    cout << _("Error: Invalid Value for max_join_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_join_size");
   }
   global_system_variables.max_join_size= in_max_join_size;
 }
@@ -813,8 +800,7 @@
   global_system_variables.max_length_for_sort_data= 1024;
   if (in_max_length_for_sort_data < 4 || in_max_length_for_sort_data > 8192*1024L)
   {
-    cout << _("Error: Invalid Value for max_length_for_sort_data");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_length_for_sort_data");
   }
   global_system_variables.max_length_for_sort_data= in_max_length_for_sort_data;
 }
@@ -824,8 +810,7 @@
   global_system_variables.max_seeks_for_key= ULONG_MAX;
   if (in_max_seeks_for_key < 1 || in_max_seeks_for_key > ULONG_MAX)
   {
-    cout << _("Error: Invalid Value for max_seeks_for_key");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_seeks_for_key");
   }
   global_system_variables.max_seeks_for_key= in_max_seeks_for_key;
 }
@@ -835,8 +820,7 @@
   global_system_variables.max_sort_length= 1024;
   if ((int64_t)in_max_sort_length < 4 || (int64_t)in_max_sort_length > 8192*1024L)
   {
-    cout << _("Error: Invalid Value for max_sort_length");
-    exit(-1);
+    unireg_abort << _("Invalid Value for max_sort_length");
   }
   global_system_variables.max_sort_length= in_max_sort_length;
 }
@@ -846,8 +830,7 @@
   global_system_variables.optimizer_search_depth= 0;
   if (in_optimizer_search_depth > MAX_TABLES + 2)
   {
-    cout << _("Error: Invalid Value for optimizer_search_depth");
-    exit(-1);
+    unireg_abort << _("Invalid Value for optimizer_search_depth");
   }
   global_system_variables.optimizer_search_depth= in_optimizer_search_depth;
 }
@@ -857,8 +840,7 @@
   global_system_variables.preload_buff_size= (32*1024L);
   if (in_preload_buff_size < 1024 || in_preload_buff_size > 1024*1024*1024L)
   {
-    cout << _("Error: Invalid Value for preload_buff_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for preload_buff_size");
   }
   global_system_variables.preload_buff_size= in_preload_buff_size;
 }
@@ -868,8 +850,7 @@
   global_system_variables.query_alloc_block_size= QUERY_ALLOC_BLOCK_SIZE;
   if (in_query_alloc_block_size < 1024)
   {
-    cout << _("Error: Invalid Value for query_alloc_block_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for query_alloc_block_size");
   }
   in_query_alloc_block_size-= in_query_alloc_block_size % 1024;
   global_system_variables.query_alloc_block_size= in_query_alloc_block_size;
@@ -880,8 +861,7 @@
   global_system_variables.query_prealloc_size= QUERY_ALLOC_PREALLOC_SIZE;
   if (in_query_prealloc_size < QUERY_ALLOC_PREALLOC_SIZE)
   {
-    cout << _("Error: Invalid Value for query_prealloc_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for query_prealloc_size");
   }
   in_query_prealloc_size-= in_query_prealloc_size % 1024;
   global_system_variables.query_prealloc_size= in_query_prealloc_size;
@@ -892,8 +872,7 @@
   global_system_variables.range_alloc_block_size= RANGE_ALLOC_BLOCK_SIZE;
   if (in_range_alloc_block_size < RANGE_ALLOC_BLOCK_SIZE)
   {
-    cout << _("Error: Invalid Value for range_alloc_block_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for range_alloc_block_size");
   }
   in_range_alloc_block_size-= in_range_alloc_block_size % 1024;
   global_system_variables.range_alloc_block_size= in_range_alloc_block_size;
@@ -904,8 +883,7 @@
   global_system_variables.read_buff_size= (128*1024L);
   if (in_read_buff_size < IO_SIZE*2 || in_read_buff_size > INT32_MAX)
   {
-    cout << _("Error: Invalid Value for read_buff_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for read_buff_size");
   }
   in_read_buff_size-= in_read_buff_size % IO_SIZE;
   global_system_variables.read_buff_size= in_read_buff_size;
@@ -916,8 +894,7 @@
   global_system_variables.read_rnd_buff_size= (256*1024L);
   if (in_read_rnd_buff_size < 64 || in_read_rnd_buff_size > UINT32_MAX)
   {
-    cout << _("Error: Invalid Value for read_rnd_buff_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for read_rnd_buff_size");
   }
   global_system_variables.read_rnd_buff_size= in_read_rnd_buff_size;
 }
@@ -927,8 +904,7 @@
   global_system_variables.sortbuff_size= MAX_SORT_MEMORY;
   if ((uint32_t)in_sortbuff_size < MIN_SORT_MEMORY)
   {
-    cout << _("Error: Invalid Value for sort_buff_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for sort_buff_size");
   }
   global_system_variables.sortbuff_size= in_sortbuff_size;
 }
@@ -938,8 +914,7 @@
   table_def_size= 128;
   if (in_table_def_size < 1 || in_table_def_size > 512*1024L)
   {
-    cout << _("Error: Invalid Value for table_def_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for table_def_size");
   }
   table_def_size= in_table_def_size;
 }
@@ -949,8 +924,7 @@
   table_cache_size= TABLE_OPEN_CACHE_DEFAULT;
   if (in_table_cache_size < TABLE_OPEN_CACHE_MIN || in_table_cache_size > 512*1024L)
   {
-    cout << _("Error: Invalid Value for table_cache_size");
-    exit(-1);
+    unireg_abort << _("Invalid Value for table_cache_size");
   }
   table_cache_size= in_table_cache_size;
 }
@@ -960,8 +934,7 @@
   table_lock_wait_timeout= 50;
   if (in_table_lock_wait_timeout < 1 || in_table_lock_wait_timeout > 1024*1024*1024)
   {
-    cout << _("Error: Invalid Value for table_lock_wait_timeout");
-    exit(-1);
+    unireg_abort <<  _("Invalid Value for table_lock_wait_timeout");
   }
   table_lock_wait_timeout= in_table_lock_wait_timeout;
 }
@@ -976,8 +949,7 @@
   global_system_variables.tmp_table_size= 16*1024*1024L;
   if (in_tmp_table_size < 1024 || in_tmp_table_size > MAX_MEM_TABLE_SIZE)
   {
-    cout << _("Error: Invalid Value for table_lock_wait_timeout");
-    exit(-1);
+    unireg_abort << _("Invalid Value for table_lock_wait_timeout");
   }
   global_system_variables.tmp_table_size= in_tmp_table_size;
 }
@@ -987,8 +959,7 @@
   transaction_message_threshold= 1024*1024;
   if ((int64_t) in_transaction_message_threshold < 128*1024 || (int64_t)in_transaction_message_threshold > 1024*1024)
   {
-    cout << _("Error: Invalid Value for transaction_message_threshold valid values are between 131072 - 1048576 bytes");
-    exit(-1);
+    unireg_abort << _("Invalid Value for transaction_message_threshold valid values are between 131072 - 1048576 bytes");
   }
   transaction_message_threshold= in_transaction_message_threshold;
 }
@@ -1024,16 +995,17 @@
   {
     fs::path p(it);
     if (fs::is_regular_file(p))
+    {
       defaults_file_list.push_back(it);
+    }
     else
     {
-      errmsg_printf(error::ERROR, _("Defaults file '%s' not found\n"), it.c_str());
-      unireg_abort(1);
+      unireg_abort << "Defaults file '" << it << "' not found";
     }
   }
 }
 
-int init_basic_variables(int argc, char **argv)
+bool init_variables_before_daemonizing(int argc, char **argv)
 {
   umask(((~internal::my_umask) & 0666));
   decimal_zero.set_zero(); // set decimal_zero constant;
@@ -1041,7 +1013,9 @@
 
   time_t curr_time= time(NULL);
   if (curr_time == (time_t)-1)
-    return 1;
+  {
+    return false;
+  }
 
   max_system_variables.pseudo_thread_id= UINT32_MAX;
   server_start_time= flush_status_time= curr_time;
@@ -1276,8 +1250,7 @@
   }
   catch (std::exception&)
   {
-    errmsg_printf(error::ERROR, _("Duplicate entry for command line option\n"));
-    unireg_abort(1);
+    unireg_abort << _("Duplicate entry for command line option");
   }
 
   if (not vm["no-defaults"].as<bool>())
@@ -1310,8 +1283,7 @@
   }
   catch (po::validation_error &err)
   {
-    errmsg_printf(error::ERROR, _("%s: %s.\nUse --help to get a list of available options\n"), internal::my_progname, err.what());
-    unireg_abort(1);
+    unireg_abort << "Use --help to get a list of available options. " << err.what(); 
   }
 
   process_defaults_files();
@@ -1325,14 +1297,15 @@
   }
   catch (po::validation_error &err)
   {
-    errmsg_printf(error::ERROR, _("%s: %s.\nUse --help to get a list of available options\n"), internal::my_progname, err.what());
-    unireg_abort(1);
+    unireg_abort << "Use --help to get a list of available options. " << err.what();
   }
 
-  return 0;
+  return true;
 }
 
-int init_remaining_variables(module::Registry &plugins)
+// Return failure if we can't pass this, unireg_abort() will then be called
+// by the caller.
+bool init_variables_after_daemonizing(module::Registry &plugins)
 {
   int style = po::command_line_style::default_style & ~po::command_line_style::allow_guessing;
 
@@ -1344,8 +1317,7 @@
 
   if (plugin_init(plugins, plugin_options))
   {
-    errmsg_printf(error::ERROR, _("Failed to initialize plugins\n"));
-    unireg_abort(1);
+    unireg_abort << _("Failed to initialize plugins");
   }
 
   full_options.add(plugin_options);
@@ -1365,26 +1337,15 @@
   }
   catch (po::validation_error &err)
   {
-    errmsg_printf(error::ERROR,
-                  _("%s: %s.\n"
-                    "Use --help to get a list of available options\n"),
-                  internal::my_progname, err.what());
-    unireg_abort(1);
+    unireg_abort << "Use --help to get a list of available options. " << err.what();
   }
   catch (po::invalid_command_line_syntax &err)
   {
-    errmsg_printf(error::ERROR,
-                  _("%s: %s.\n"
-                    "Use --help to get a list of available options\n"),
-                  internal::my_progname, err.what());
-    unireg_abort(1);
+    unireg_abort << "Use --help to get a list of available options. " << err.what();
   }
   catch (po::unknown_option &err)
   {
-    errmsg_printf(error::ERROR,
-                  _("%s\nUse --help to get a list of available options\n"),
-                  err.what());
-    unireg_abort(1);
+    unireg_abort << "Use --help to get a list of available options. " << err.what();
   }
 
   try
@@ -1393,11 +1354,7 @@
   }
   catch (po::validation_error &err)
   {
-    errmsg_printf(error::ERROR,
-                  _("%s: %s.\n"
-                    "Use --help to get a list of available options\n"),
-                  internal::my_progname, err.what());
-    unireg_abort(1);
+    unireg_abort << "Use --help to get a list of available options. " << err.what();
   }
 
   get_options();
@@ -1420,38 +1377,23 @@
 
   item_create_init();
   if (sys_var_init())
-    return 1;
+  {
+    return false;
+  }
+
   /* Creates static regex matching for temporal values */
   if (not init_temporal_formats())
-    return 1;
+    return false;
 
-  if (!(default_charset_info=
-        get_charset_by_csname(default_character_set_name, MY_CS_PRIMARY)))
+  if (!(default_charset_info= get_charset_by_csname(default_character_set_name, MY_CS_PRIMARY)))
   {
     errmsg_printf(error::ERROR, _("Error getting default charset"));
-    return 1;                           // Eof of the list
+    return false;                           // Eof of the list
   }
 
   if (vm.count("scheduler"))
     opt_scheduler= vm["scheduler"].as<string>().c_str();
 
-  if (default_collation_name)
-  {
-    const charset_info_st * const default_collation= get_charset_by_name(default_collation_name);
-    if (not default_collation)
-    {
-      errmsg_printf(error::ERROR, _(ER(ER_UNKNOWN_COLLATION)), default_collation_name);
-      return 1;
-    }
-    if (not my_charset_same(default_charset_info, default_collation))
-    {
-      errmsg_printf(error::ERROR, _(ER(ER_COLLATION_CHARSET_MISMATCH)),
-                    default_collation_name,
-                    default_charset_info->csname);
-      return 1;
-    }
-    default_charset_info= default_collation;
-  }
   /* Set collactions that depends on the default collation */
   global_system_variables.collation_server=	 default_charset_info;
 
@@ -1459,22 +1401,21 @@
            get_charset_by_csname(character_set_filesystem_name, MY_CS_PRIMARY)))
   {
     errmsg_printf(error::ERROR, _("Error setting collation"));
-    return 1;
+    return false;
   }
   global_system_variables.character_set_filesystem= character_set_filesystem;
 
-  if (!(my_default_lc_time_names=
-        my_locale_by_name(lc_time_names_name)))
+  if ((my_default_lc_time_names= my_locale_by_name(lc_time_names_name)) == NULL)
   {
     errmsg_printf(error::ERROR, _("Unknown locale: '%s'"), lc_time_names_name);
-    return 1;
+    return false;
   }
   global_system_variables.lc_time_names= my_default_lc_time_names;
 
   /* Reset table_alias_charset */
   table_alias_charset= files_charset_info;
 
-  return 0;
+  return true;
 }
 
 
@@ -1496,17 +1437,18 @@
   ha_init_errors();
 
   if (opt_help)
-    unireg_abort(0);
+  {
+    unireg_exit();
+  }
 
   if (plugin_finalize(plugins))
   {
-    unireg_abort(1);
+    unireg_abort << "plugin_finalize() failed";
   }
 
   if (plugin::Scheduler::setPlugin(opt_scheduler))
   {
-      errmsg_printf(error::ERROR, _("No scheduler found, cannot continue!\n"));
-      unireg_abort(1);
+    unireg_abort << _("No scheduler found");
   }
 
   /*
@@ -1524,8 +1466,7 @@
     plugin::StorageEngine *engine= plugin::StorageEngine::findByName(default_storage_engine_str);
     if (engine == NULL)
     {
-      errmsg_printf(error::ERROR, _("Unknown/unsupported storage engine: %s\n"), default_storage_engine_str);
-      unireg_abort(1);
+      unireg_abort << _("Unknown/unsupported storage engine: ") << default_storage_engine_str;
     }
     global_system_variables.storage_engine= engine;
   }
@@ -1533,7 +1474,7 @@
   if (plugin::XaResourceManager::recoverAllXids())
   {
     /* This function alredy generates error messages */
-    unireg_abort(1);
+    unireg_abort << "plugin::XaResourceManager::recoverAllXids() failed";
   }
 
   init_update_queries();
@@ -1638,10 +1579,6 @@
    N_("Chroot drizzled daemon during startup."),
    (char**) &drizzled_chroot, NULL, 0, GET_STR, REQUIRED_ARG,
    0, 0, 0, 0, 0, 0},
-  {"collation-server", OPT_DEFAULT_COLLATION,
-   N_("Set the default collation."),
-   (char**) &default_collation_name, NULL,
-   0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   {"completion-type", OPT_COMPLETION_TYPE,
    N_("Default completion type."),
    (char**) &global_system_variables.completion_type,
@@ -1909,12 +1846,18 @@
 
 static void usage()
 {
-  if (!(default_charset_info= get_charset_by_csname(default_character_set_name, MY_CS_PRIMARY)))
-    exit(1);
-  if (!default_collation_name)
+  if ((default_charset_info= get_charset_by_csname(default_character_set_name, MY_CS_PRIMARY)) == NULL)
+  {
+    unireg_abort << "Failed to load default_charset_info:" << default_character_set_name;
+  }
+
+  if (default_collation_name == NULL)
+  {
     default_collation_name= default_charset_info->name;
+  }
+
   print_version();
-  puts(_("Copyright (C) 2008 Sun Microsystems\n"
+  puts(_("Copyright (C) 2010-2011 Drizzle Developers, Copyright (C) 2008 Sun Microsystems\n"
          "This software comes with ABSOLUTELY NO WARRANTY. "
          "This is free software,\n"
          "and you are welcome to modify and redistribute it under the GPL "
@@ -2038,17 +1981,15 @@
   if (vm.count("version"))
   {
     print_version();
-    exit(0);
+    unireg_exit();
   }
 
   if (vm.count("sort-heap-threshold"))
   {
     if ((vm["sort-heap-threshold"].as<uint64_t>() > 0) and
-      (vm["sort-heap-threshold"].as<uint64_t>() <
-      global_system_variables.sortbuff_size))
+      (vm["sort-heap-threshold"].as<uint64_t>() < global_system_variables.sortbuff_size))
     {
-      cout << _("Error: sort-heap-threshold cannot be less than sort-buffer-size") << endl;
-      exit(-1);
+      unireg_abort << _("sort-heap-threshold cannot be less than sort-buffer-size");
     }
 
     global_sort_buffer.setMaxSize(vm["sort-heap-threshold"].as<uint64_t>());
@@ -2057,11 +1998,9 @@
   if (vm.count("join-heap-threshold"))
   {
     if ((vm["join-heap-threshold"].as<uint64_t>() > 0) and
-      (vm["join-heap-threshold"].as<uint64_t>() <
-      global_system_variables.join_buff_size))
+      (vm["join-heap-threshold"].as<uint64_t>() < global_system_variables.join_buff_size))
     {
-      cout << _("Error: join-heap-threshold cannot be less than join-buffer-size") << endl;
-      exit(-1);
+      unireg_abort << _("join-heap-threshold cannot be less than join-buffer-size");
     }
 
     global_join_buffer.setMaxSize(vm["join-heap-threshold"].as<uint64_t>());
@@ -2070,11 +2009,9 @@
   if (vm.count("read-rnd-threshold"))
   {
     if ((vm["read-rnd-threshold"].as<uint64_t>() > 0) and
-      (vm["read-rnd-threshold"].as<uint64_t>() <
-      global_system_variables.read_rnd_buff_size))
+      (vm["read-rnd-threshold"].as<uint64_t>() < global_system_variables.read_rnd_buff_size))
     {
-      cout << _("Error: read-rnd-threshold cannot be less than read-rnd-buffer-size") << endl;
-      exit(-1);
+      unireg_abort << _("read-rnd-threshold cannot be less than read-rnd-buffer-size");
     }
 
     global_read_rnd_buffer.setMaxSize(vm["read-rnd-threshold"].as<uint64_t>());
@@ -2083,11 +2020,9 @@
   if (vm.count("read-buffer-threshold"))
   {
     if ((vm["read-buffer-threshold"].as<uint64_t>() > 0) and
-      (vm["read-buffer-threshold"].as<uint64_t>() <
-      global_system_variables.read_buff_size))
+      (vm["read-buffer-threshold"].as<uint64_t>() < global_system_variables.read_buff_size))
     {
-      cout << _("Error: read-buffer-threshold cannot be less than read-buffer-size") << endl;
-      exit(-1);
+      unireg_abort << _("read-buffer-threshold cannot be less than read-buffer-size");
     }
 
     global_read_buffer.setMaxSize(vm["read-buffer-threshold"].as<uint64_t>());
@@ -2113,7 +2048,22 @@
 
   if (vm.count("skip-symlinks"))
   {
-    internal::my_use_symdir=0;
+    internal::my_use_symdir= 0;
+  }
+
+  if (vm.count("collation-server"))
+  {
+    const charset_info_st * const default_collation= get_charset_by_name(vm["collation-server"].as<string>().c_str());
+    if (not default_collation)
+    {
+      unireg_abort << "Unknown collation: " << default_collation_name;
+    }
+
+    if (not my_charset_same(default_charset_info, default_collation))
+    {
+      unireg_abort << "COLLATION '" << default_collation_name << "' is not valid for CHARACTER SET '" << default_charset_info->csname << "'";
+    }
+    default_charset_info= default_collation;
   }
 
   if (vm.count("transaction-isolation"))
@@ -2149,7 +2099,9 @@
   }
 
   if (drizzled_chroot)
+  {
     set_root(drizzled_chroot);
+  }
 
   /*
     Set some global variables from the global_system_variables
@@ -2197,18 +2149,19 @@
     {
       if (errno != EEXIST)
       {
-        errmsg_printf(error::ERROR, _("There was an error creating the '%s' part of the path '%s'.  Please check the path exists and is writable.\n"), fs::path(drizzle_tmpdir).leaf().c_str(), drizzle_tmpdir.c_str());
-        exit(1);
+        unireg_abort << "There was an error creating the '" 
+          << fs::path(drizzle_tmpdir).leaf() 
+          << "' part of the path '" 
+          << drizzle_tmpdir 
+          << "'.  Please check the path exists and is writable.";
       }
     }
 
     if (stat(drizzle_tmpdir.c_str(), &buf) || not S_ISDIR(buf.st_mode))
     {
-      errmsg_printf(error::ERROR, _("There was an error opening the path '%s', please check the path exists and is writable.\n"), drizzle_tmpdir.c_str());
-      exit(1);
+      unireg_abort << "There was an error opening the path '" << drizzle_tmpdir << "', please check the path exists and is writable.";
     }
   }
-
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/drizzled.h'
--- drizzled/drizzled.h	2011-06-22 21:57:14 +0000
+++ drizzled/drizzled.h	2011-07-18 02:56:15 +0000
@@ -53,8 +53,8 @@
 
 int init_thread_environment();
 void init_server_components(module::Registry&);
-int init_basic_variables(int argc, char **argv);
-int init_remaining_variables(module::Registry&);
+bool init_variables_before_daemonizing(int argc, char **argv);
+bool init_variables_after_daemonizing(module::Registry&);
 
 passwd *check_user(const char *user);
 void set_user(const char *user, passwd *user_info_arg);

=== modified file 'drizzled/errmsg_print.cc'
--- drizzled/errmsg_print.cc	2011-03-29 09:33:49 +0000
+++ drizzled/errmsg_print.cc	2011-07-18 02:56:15 +0000
@@ -85,7 +85,7 @@
   errmsg_printf(error::ERROR, "%s: %s\n", message.c_str(), errmsg_ptr);
 }
 
-bool errmsg_printf (error::level_t priority, char const *format, ...)
+bool errmsg_printf(error::level_t priority, char const *format, ...)
 {
   bool rv;
   va_list args;

=== modified file 'drizzled/main.cc'
--- drizzled/main.cc	2011-06-24 13:24:06 +0000
+++ drizzled/main.cc	2011-07-18 02:56:15 +0000
@@ -252,10 +252,11 @@
   /* Function generates error messages before abort */
   error_handler_hook= my_message_sql;
 
-  /* init_common_variables must get basic settings such as data_home_dir
-     and plugin_load_list. */
-  if (init_basic_variables(argc, argv))
-    unireg_abort(1);				// Will do exit
+  /* init_common_variables must get basic settings such as data_home_dir and plugin_load_list. */
+  if (not init_variables_before_daemonizing(argc, argv))
+  {
+    unireg_abort << "init_variables_before_daemonizing() failed";				// Will do exit
+  }
 
   if (opt_daemon)
   {
@@ -265,13 +266,15 @@
     }
     if (daemonize())
     {
-      fprintf(stderr, "failed to daemon() in order to daemonize\n");
-      exit(EXIT_FAILURE);
+      unireg_abort << "--daemon failed";
     }
   }
 
-  if (init_remaining_variables(modules))
-    unireg_abort(1);				// Will do exit
+  if (not init_variables_after_daemonizing(modules))
+  {
+    unireg_abort << "init_variables_after_daemonizing() failed";				// Will do exit
+  }
+
 
   /*
     init signals & alarm
@@ -287,16 +290,13 @@
   {
     if (chdir(getDataHome().file_string().c_str()))
     {
-      errmsg_printf(error::ERROR,
-                    _("Data directory %s does not exist\n"),
-                    getDataHome().file_string().c_str());
-      unireg_abort(1);
+      unireg_abort << "Data directory " << getDataHome().file_string() << " does not exist";
     }
 
     ifstream old_uuid_file ("server.uuid");
     if (old_uuid_file.is_open())
     {
-      getline (old_uuid_file, server_uuid);
+      getline(old_uuid_file, server_uuid);
       old_uuid_file.close();
     } 
     else 
@@ -311,16 +311,30 @@
       server_uuid= string(uuid_string);
     }
 
-    if (mkdir("local", 0700))
+    if (mkdir("local", 0700) == -1)
     {
-      /* We don't actually care */
+      switch (errno)
+      {
+      case EEXIST:
+        break;
+
+      case EACCES:
+        {
+          char cwd[1024];
+          unireg_abort << "Could not create local catalog, permission denied in directory:" << getcwd(cwd, sizeof(cwd));
+        }
+
+      default:
+        {
+          char cwd[1024];
+          unireg_abort << "Could not create local catalog, in directory:" << getcwd(cwd, sizeof(cwd)) << " system error was:" << strerror(errno);
+        }
+      }
     }
-    if (chdir("local"))
+
+    if (chdir("local") == -1)
     {
-      errmsg_printf(error::ERROR,
-                    _("Local catalog %s/local does not exist\n"),
-                    getDataHome().file_string().c_str());
-      unireg_abort(1);
+      unireg_abort << "Local catalog does not exist, was unable to chdir() to " << getDataHome().file_string();
     }
 
     setFullDataHome(boost::filesystem::system_complete(getDataHome()));
@@ -344,7 +358,7 @@
     cout << _("In File: ") << *::boost::get_error_info<boost::throw_file>(ex) << endl;
     cout << _("On Line: ") << *::boost::get_error_info<boost::throw_line>(ex) << endl;
 #endif
-    unireg_abort(1);
+    unireg_abort << "init_server_components() failed";
   }
 
 
@@ -362,7 +376,9 @@
   (void) ReplicationServices::evaluateRegisteredPlugins();
 
   if (plugin::Listen::setup())
-    unireg_abort(1);
+  {
+    unireg_abort << "Failed plugin::Listen::setup()";
+  }
 
   assert(plugin::num_trx_monitored_objects > 0);
   drizzle_rm_tmp_tables();

=== modified file 'drizzled/module/registry.h'
--- drizzled/module/registry.h	2011-04-20 12:02:09 +0000
+++ drizzled/module/registry.h	2011-07-18 02:56:15 +0000
@@ -97,18 +97,27 @@
     std::string plugin_name(boost::to_lower_copy(plugin->getName()));
     if (find_ptr(plugin_registry, std::make_pair(plugin_type, plugin_name)))
     {
-      errmsg_printf(error::ERROR, _("Loading plugin %s failed: a %s plugin by that name already exists.\n"), 
-        plugin->getTypeName().c_str(), plugin->getName().c_str());
-      failed= true;
-    }
-    if (T::addPlugin(plugin)) // Olaf: Should addPlugin be called when failed is already true?
-      failed= true; 
+      std::string error_message;
+      error_message+=  _("Loading plugin failed, a plugin by that name already exists.");
+      error_message+= plugin->getTypeName();
+      error_message+= ":";
+      error_message+= plugin->getName();
+      unireg_actual_abort(__FILE__, __LINE__, __func__, error_message);
+    }
+
+    if (T::addPlugin(plugin))
+    {
+      std::string error_message;
+      error_message+= _("Fatal error: Failed initializing: ");
+      error_message+= plugin->getTypeName();
+      error_message+= ":";
+      error_message+= plugin->getName();
+      unireg_actual_abort(__FILE__, __LINE__, __func__, error_message);
+    }
 
     if (failed)
     {
-      errmsg_printf(error::ERROR, _("Fatal error: Failed initializing %s::%s plugin.\n"), 
-        plugin->getTypeName().c_str(), plugin->getName().c_str());
-      unireg_abort(1);
+      unireg_abort << _("Fatal error: Failed initializing: ") << plugin->getTypeName() << ":" << plugin->getName();
     }
     plugin_registry.insert(std::make_pair(std::make_pair(plugin_type, plugin_name), plugin));
   }

=== modified file 'drizzled/unireg.h'
--- drizzled/unireg.h	2011-03-14 05:40:28 +0000
+++ drizzled/unireg.h	2011-07-18 02:56:15 +0000
@@ -23,11 +23,82 @@
 #pragma once
 
 #include <drizzled/visibility.h>
+#include <sstream>
 
 namespace drizzled
 {
 
-DRIZZLED_API void unireg_abort(int exit_code) __attribute__((noreturn));
+void unireg_exit() __attribute__((noreturn));
+DRIZZLED_API void unireg_actual_abort(const char *file, int line, const char *func, const std::string& message) __attribute__((noreturn));
+
+namespace stream {
+
+namespace detail {
+
+template<class Ch, class Tr, class A>
+  class _unireg {
+  private:
+
+  public:
+    typedef std::basic_ostringstream<Ch, Tr, A> stream_buffer;
+
+  public:
+    void operator()(const stream_buffer &s, const char *filename, int line, const char *func)
+    {
+      unireg_actual_abort(filename, line, func, s.str());
+    }
+  };
+
+template<template <class Ch, class Tr, class A> class OutputPolicy, class Ch = char, class Tr = std::char_traits<Ch>, class A = std::allocator<Ch> >
+  class log {
+  private:
+    typedef OutputPolicy<Ch, Tr, A> output_policy;
+    const char *_filename;
+    int _line_number;
+    const char *_func;
+
+  public:
+    log() :
+      _filename(NULL),
+      _line_number(0),
+      _func(NULL)
+    { }
+
+    void set_filename(const char *filename, int line_number, const char *func)
+    {
+      _filename= filename;
+      _line_number= line_number;
+      _func= func;
+    }
+
+    ~log()
+    {
+      output_policy()(arg, _filename, _line_number, _func);
+    }
+
+  public:
+    template<class T>
+      log &operator<<(const T &x)
+      {
+        arg << x;
+        return *this;
+      }
+
+  private:
+    typename output_policy::stream_buffer arg;
+  };
+} // namespace detail
+
+class _unireg : public detail::log<detail::_unireg> {
+public:
+  _unireg(const char *filename, int line_number, const char *func)
+  {
+    set_filename(filename, line_number, func);
+  }
+};
+
+} // namespace stream
+
+#define unireg_abort stream::_unireg(__FILE__, __LINE__, __func__)
 
 } /* namespace drizzled */
-

